from .core._parser import InfoTypes, ResultTypes


def dump_search(data, pretty=False):
    for type_, id_, texts, link in data:
        type_ = ResultTypes.tostring(type_) if pretty else type_
        link_ = _shorten(link, 50) if pretty else link
        print(f'{type_} {id_} {link_}')
        for text in texts:
            print(f'  {_shorten(text, 70)}')


def dump_info(data, pretty=False):
    name, image_link, texts = data
    image_link_ = _shorten(image_link, 70) if pretty else image_link
    print(f'{name}\nImage: {image_link_}')
    for type_, descs in texts:
        print(f'  {type_} {descs}')

def _shorten(text, max_len):
    if len(text) > max_len:
        return f'{text[:50]}...'
    else:
        return text


from abc import abstractmethod


class BrowserWrapper:
    def open(self, browser, url):
        return browser.open(url)
    
    def submit_selected(self, browser):
        return browser.submit_selected()


__all__ = [
    'InfoTypes',
    'ResultTypes'
]


class InfoTypes:
    WORK = 0
    EDUCATION = 1
    LIVING = 2

    def __init__(self):
        pass

    @staticmethod
    def tostring(type_):
        if type_ == InfoTypes.WORK:
            return 'work'
        elif type_ == InfoTypes.EDUCATION:
            return 'education'
        elif type_ == InfoTypes.LIVING:
            return 'living'

    @staticmethod
    def fromstring(str_):
        if str_ == 'work':
            return InfoTypes.WORK
        elif str_ == 'education':
            return InfoTypes.EDUCATION
        elif str_ == 'living':
            return InfoTypes.LIVING


class ResultTypes:
    PROFILE = 0
    GROUP = 1
    EVENT = 2
    VIDEO = 3

    def __init__(self):
        pass

    @staticmethod
    def tostring(type_):
        if type_ == ResultTypes.PROFILE:
            return 'profile'
        elif type_ == ResultTypes.GROUP:
            return 'group'
        elif type_ == ResultTypes.EVENT:
            return 'event'
        elif type_ == ResultTypes.VIDEO:
            return 'video'


def parse_image(page, name):
    image_link = ''
    image = page.select_one(f"img[alt='{name}']")
    if image:
        image_link = image.attrs['src'] if 'src' in image.attrs else ''
    return image_link


def parse_info(page):
    items = \
        _parse_info(page, 'work')
    items.extend(
        _parse_info(page, 'education'))
    items.extend(
        _parse_info(page, 'living'))
    return items


def parse_search(page, base_url):
    container = page.select_one('div#BrowseResultsContainer')
    if not container:
        return []
    results = []
    for a in container.find_all('a'):
        if 'href' in a.attrs:
            href = a.attrs['href']
            type_ = _get_profile_type(href)
            id_ = _get_profile_id(href) \
                if type_ == ResultTypes.PROFILE else None
            link = href if 'http' in href else f'{base_url}{href}'
            texts = []
            # Video lacks profile ID and descriptions
            if type_ != ResultTypes.VIDEO:          
                for div in a.find_all('div'):
                    text = div.get_text()
                    # Avoid duplicates
                    if len(text) > 0 and text not in texts:
                        texts.append(text)
            if len(texts) > 0 or type_ == ResultTypes.VIDEO:
                results.append((type_, id_, texts, link))
    return results


def _parse_info(page, type_):
    infos = []
    div = page.select_one(f'div#{type_}')
    if div:
        for link in div.find_all('a'):
            text = link.get_text()
            if text:
                infos.append((InfoTypes.fromstring(type_), text))
    return infos


def _get_profile_id(uri_part):
    import re
    # m.facebook.com/profile.php?id=[profile.id]
    # m.facebook.com/profile.php?id=[profile.id]?refid=n
    # m.facebook.com/profile.php?id=[profile.id]&refid=n
    matches = re.findall(r'(?<=\?id=).+?(?=$|\?|&)', uri_part)
    if matches:
        return matches[0]

    # m.facebook.com/[profile.id]
    # m.facebook.com/[profile.id]/?refid=n
    # m.facebook.com/[profile.id]?refid=n
    # m.facebook.com/[profile.id]&refid=n
    matches = re.findall(r'(?<=/).*?(?=$|/|\?|&)', uri_part)
    if matches:
        return matches[0]
    return ''


def _get_profile_type(uri_part):
    if '/groups/' in uri_part:
        return ResultTypes.GROUP
    elif '/events/' in uri_part:
        return ResultTypes.EVENT
    elif '/video_redirect/' in uri_part:
        return ResultTypes.VIDEO
    return ResultTypes.PROFILE


from .__version__ import __version__
from .core.session import Session
from .core.wrapper import BrowserWrapper
from .core.exceptions import LogInError, NotConnectedError
from .core._parser import InfoTypes, ResultTypes


def init_session():
    return Session(BrowserWrapper())


import os.path
from abc import abstractmethod
from mechanicalsoup import StatefulBrowser

from .exceptions import LogInError, NotConnectedError
from .wrapper import BrowserWrapper
from ._parser import parse_image, parse_info, parse_search


__all__ = [
    "Session"
]


class Session:
    BASE_URL = 'https://m.facebook.com'

    def __init__(self, browser_wrapper):
        self._connected = False
        self._current_html = None
        self._browser_wrapper = browser_wrapper
        self._browser = StatefulBrowser()
        self._browser.addHeaders = [
                ('User-Agent', 'Firefox'),
                ('Accept-Language', 'en-US,en;q=0.5')
                ]

    def __del__(self):
        self._dispose()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._dispose()

    @property
    def connected(self):
        return self._connected

    def log_in(self, username, password):
        try:
            # Log in to non-mobile site is more reliable
            self._browser_wrapper.open(self._browser, 'https://www.facebook.com')
            self._browser.select_form('div[class="_6lux"]:has(> input[name="email"])')
            self._browser['email'] = username
            self._browser.select_form('div[class="_6lux"]:has(> input[name="pass"])')
            self._browser['pass'] =  password
            self._browser.select_form('div[class="_6ltg"]')        
            self._browser_wrapper.submit_selected(self._browser)
            # Check if we really are in account profile page
            if self._browser.get_current_page().find('form',
                action='/search/top/'):
                self._connected = True
        except:
            raise LogInError(f'Unable to log in as {username}')
        return self

    def log_out(self):
        if self._connected:
            self._browser.close()
            self._connected = False

    def profile_info(self, id_):
        """Retrieve informations for a given profile."""
        self._ensure_connected()
        try:
            self._browser_wrapper.open(self._browser, f'{Session.BASE_URL}/{id_}')
            name  = self._sanitize_title(
                self._browser.get_current_page().find('title').text)
            image = parse_image(self._browser.get_current_page(), name)
            info = parse_info(self._browser.get_current_page())
            return name, image, info
        except:
            return None

    def search(self, query):
        """
        Execute search of a given text returning a tuple with ID,
        descriptions and URI.
        """
        url_query = '+'.join(query.split())
        url_path = f'/search/top/?q={url_query}' \
            if self._connected else f'/public/{url_query}'
        try:
            self._browser_wrapper.open(self._browser,
                f'{Session.BASE_URL}{url_path}{url_query}')
            return parse_search(self._browser.get_current_page(),
                Session.BASE_URL)
        except:
            return None

    def _ensure_connected(self):
        if not self._connected:
            raise NotConnectedError('No active connection or required login')

    def _sanitize_title(self, title):
        # Handle cases like 'Some One - Home'
        if '-' in title:
            return title.split('-')[0].strip()
        return title

    def _dispose(self):
        if self._connected:
            self.log_out()



# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

# Package meta-data
NAME = 'facebook-snooper'
DESCRIPTION = 'Scrapes Facebook profile pages and searches profiles using a normal log in (without an API key).'
URL = 'https://github.com/gsscoder/facebook-snooper'
EMAIL = 'gsscoder@gmail.com'
AUTHOR = 'Giacomo Stelluti Scala'
VERSION = '0.5.14'

# Required packages
REQUIRED = [
    'MechanicalSoup'
]

setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*", "dist", "*.egg-info"]),
    install_requires=REQUIRED,
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7'
        ]
)
